/*
  Extension       Test_Graphics_around_Text
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   Windows GUI TSE, screen not maximized, font Courier New
  Version         v0.1   28 Dec 2023

  This extension tests drawing around text.

  Compatibility:
    It will never be compatible with Linux TSE and Windows Console TSE.
    It will probably never be compatible with a maximized TSE screen.
    It is currently only compatible with the Courier New font.

  Installation:
    Compile it and execute or (auto)load it.

  Usage:
    Press the <Alt> key by itself.
      This will currently show all the pixels it can draw around letters
      as yellow dots on a blue background.
    Afterwards any other key will restore the screen and perform the key's
    regular function.

  Limitations:
  - Drawing is slow.
    In my context it takes over 6 seconds to draw a whole screen.
    So this functionality will only be useful in applications that need little
    drawing or where drawing time is less relevant.
  - Pixels are big.
    In my context a text character is as wide as 6.4 pixels and as high as 7.6
    pixels.

  Known side effect:
    The extension might resize the screen a few pixels.

  Note:
    Drawing around letters is a feature, not a limitation.
    Drawing over letters would easier, but has no purpose here.
    Especially because here drawing over a letter cannot be undone per pixel,
    only by undoing the whole drawing.
    See the "xmas" tool for an example of replacing all text with a drawn and
    slightly animated picture.

  Possible applications (in theory!):
  - Pretty vertical lines between "if" and "endif", "for" and "endfor", etc.
    I only mention this because I see it in other editors.
    My feeling is that better solution would be a "|" character in the comment
    color, which would be only a fraction less pretty and would be compatible
    across all TSE variants.
  - A new cursor: A rectangle around the cursor position.
  - A background picture.
    Every once in a while and/or after a certain key.
  - An animation.
    Every once in a while and/or after a certain key.
    Examples:
      An analogue clock.
      A weather indicator.
      A progress bar.
      A pie chart of TSE's memory usage.
      Just something fun:
        Snow.
        Fireworks.

*/




/*

  T E C H N I C A L   B A C K G R O U N D   I N F O


  ONLY AFTER A KEY STROKE

  It is not possible to start drawing when the editor is idle.


  FINDING THE MINIMUM FONT SIZE

  SetFont() always returns TRUE, so is no help here.

  Repeating ResizeFont(-1) until FALSE works, but has the side effect that it
  significantly resizes the screen.
  That said, I think the screen can be restored, and this might become a
  configuration option.


  NOT UPDATING THE WHOLE SCREEN TO THE TINY FONT

  While (in a) drawing, KeyPressed() must be within a PopWinOpen/Close() in
  order to not update the whole screen to the tiny font.


  NO COLOR LIMITATION

  In theory this extension's drawing technique falls within TSE's limitations
  for giving each pixel its own RGB color. There are 16,777,216 RGB colors.

*/





#ifdef LINUX
  This macro does not work in LINUX. It needs the Windows GUI Version of TSE.
#endif





// Global constants and semi-constants.
string MACRO_NAME  [MAXSTRINGLEN] = ''
// string CFG_SECTION [MAXSTRINGLEN] = ''


// Global variables
integer free_id                 = 0
integer font_warning_shown      = FALSE
integer maximized_warning_shown = FALSE





dll "<user32.dll>"
  integer proc IsMaximized(integer hwnd) : "IsZoomed"
end





integer proc write_profile_int(string  section_name,
                               string  item_name,
                               integer item_value)
  integer ok = WriteProfileInt(section_name,
                               item_name,
                               item_value)
  if not ok
    Warn(MACRO_NAME; 'error in writing configuration to file "tse.ini".')
    PurgeMacro(MACRO_NAME)
  endif
  return(ok)
end write_profile_int


proc test()
  string  a [MAXSTRINGLEN] = ''
  integer free_location    = 0
  integer free_locations   = 0
  integer gfx_col          = 0
  integer gfx_font_size                = 2
  integer gfx_row          = 0
  integer gfx_ScreenCols               = 0
  integer gfx_ScreenRows               = 0
  string  s [MAXSTRINGLEN] = ''
  integer t0 = 0
  integer t1 = 0
  integer txt_col          = 0
  string  txt_col_row [11] = ''
  integer txt_font_flags               = 0
  string  txt_font_name [MAXSTRINGLEN] = ''
  integer txt_font_size                = 0
  integer txt_row          = 0
  integer txt_ScreenCols               = 0
  integer txt_ScreenRows               = 0

  UpdateDisplay()

  GetFont(txt_font_name, txt_font_size, txt_font_flags)
  txt_ScreenCols = Query(ScreenCols)
  txt_ScreenRows = Query(ScreenRows)

  if txt_font_name <> 'Courier New'
    Warn(MACRO_NAME; 'currently only works with the font Courier New.')
    PurgeMacro(MACRO_NAME)
    return()
  endif

  for txt_row = 1 to txt_ScreenRows
    GetStrAttrXY(1, txt_row, s, a, txt_ScreenCols)
    for txt_col = 1 to Length(s)
      if s[txt_col] == ' '
        free_locations = free_locations + 1
        SetBufferStr('FreeLoc' + Str(free_locations),
                     Str(txt_col) + ' ' + Str(txt_row),
                     free_id)
      endif
    endfor
  endfor

  SetFont(txt_font_name, gfx_font_size, txt_font_flags)
  gfx_ScreenCols = Query(ScreenCols)
  gfx_ScreenRows = Query(ScreenRows)

  PopWinOpen(1, 1, gfx_ScreenCols, gfx_ScreenRows, 0, '', Color(Intense Bright Yellow ON Blue))

  t0 = GetTime()

  for free_location = 1 to free_locations
    txt_col_row = GetBufferStr('FreeLoc' + Str(free_location), free_id)
    txt_col = Val(GetToken(txt_col_row, ' ', 1))
    txt_row = Val(GetToken(txt_col_row, ' ', 2))

    for gfx_col = (txt_col - 1) * gfx_ScreenCols / txt_ScreenCols + 1
    to  txt_col * gfx_ScreenCols / txt_ScreenCols
      for gfx_row = (txt_row - 1) * gfx_ScreenRows / txt_ScreenRows + 1
      to  txt_row * gfx_ScreenRows / txt_ScreenRows
        PutStrAttrXY(gfx_col, gfx_row, '*', '',
                     Color(Intense Bright Yellow ON Blue))
      endfor
    endfor
    if KeyPressed()
      break
    endif
  endfor

  t1 = GetTime()

  while not KeyPressed()
  endwhile

  PopWinClose()

  SetFont(txt_font_name, txt_font_size, txt_font_flags)

  /*
  UpdateDisplay()
  Message(MACRO_NAME; 'draw time ='; t1 - t0)
  Delay(36)
  */
  t0 = t0
  t1 = t1
end test


proc do_gfx()
  test()
end do_gfx


proc after_getkey()
  integer font_flags               = 0
  string  font_name [MAXSTRINGLEN] = ''
  integer font_size                = 0

  if Query(Key) == <Alt>
    if  QueryEditState() == 0
    and not KeyPressed()
      if isMaximized(GetWinHandle())
        if not maximized_warning_shown
          Warn(MACRO_NAME; 'does not work in a maximized screen.')
          maximized_warning_shown = TRUE
        endif
      else
        maximized_warning_shown = FALSE
        GetFont(font_name, font_size, font_flags)
        if font_name == 'Courier New'
          font_warning_shown = FALSE
          do_gfx()
        else
          if not font_warning_shown
            Warn(MACRO_NAME; 'needs font Courier New instead of'; font_name, '.')
            font_warning_shown = TRUE
          endif
        endif
      endif
    endif
  endif
end after_getkey


proc WhenPurged()
  AbandonFile(free_id)
end WhenPurged


proc WhenLoaded()
  MACRO_NAME  = SplitPath(CurrMacroFilename(), _NAME_)

  if isGUI()
    PushLocation()
    free_id = CreateTempBuffer()
    PopLocation()

    Hook(_AFTER_COMMAND_, after_getkey)
  else
    if  GetProfileInt(MACRO_NAME, 'RepeatConsoleWarning', TRUE)
    and MsgBox(MACRO_NAME + ' abort',
               Format(MACRO_NAME;
                      'requires the Windows GUI version of TSE.',
                      Chr(13), Chr(13),
                      'Repeat this error message any next time?'),
                _YES_NO_) == 2
      write_profile_int(MACRO_NAME, 'RepeatConsoleWarning', FALSE)
    endif
    PurgeMacro(MACRO_NAME)
  endif
end WhenLoaded


